'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Store = undefined;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

exports.default = createStore;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _rxjs = require('rxjs');

var _toMap = require('./util/toMap');

var _toMap2 = _interopRequireDefault(_toMap);

var _createReducerForStore = require('./util/createReducerForStore');

var _createReducerForStore2 = _interopRequireDefault(_createReducerForStore);

var _Collection = require('./Collection');

var _immutable = require('immutable');

var _StoreConstants = require('./constants/StoreConstants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var missing = (0, _symbol2.default)('missingSubject');

var Store = exports.Store = function () {
  Store.isStore = function isStore(obj) {
    return (typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) === 'object' && obj instanceof Store;
  };

  function Store(identifier) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    (0, _classCallCheck3.default)(this, Store);

    (0, _invariant2.default)(typeof identifier === 'string', 'Store: `identifier` is expected to be a string.');

    this[missing] = {
      subject: new _rxjs.Subject(),
      cache: {}
    };

    this.identifier = identifier;
    this.opts = opts;
    this.dependencies = {};
    this.hooks = {};

    this.insert = this.insert.bind(this);
    this.remove = this.remove.bind(this);
    this.filter = this.filter.bind(this);
    this.update = this.update.bind(this);
  }

  Store.prototype.pre = function pre(transformer) {
    (0, _invariant2.default)(typeof transformer === 'function', 'Store: `transformer` is expected to be a function.');

    this.hooks.pre = transformer;
    return this;
  };

  Store.prototype.dependsOn = function dependsOn(identifier) {
    var dependencies = this.dependencies;


    (0, _invariant2.default)(typeof identifier === 'string', 'Store: `identifier` is expected to be a string.');
    (0, _invariant2.default)(!dependencies[identifier], 'Store: There is already an existing dependency to the store `' + identifier + '`.');

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    (0, _invariant2.default)(args.length <= 2 && args.length >= 1, 'Store: `dependsOn` is expected to receive `setter` or `getter` and `setter`.');

    var dependency = void 0;
    if (args.length === 1) {
      var setter = args[0];

      (0, _invariant2.default)(typeof setter === 'function', 'Store: `setter` is expected to be a function.');

      dependency = { identifier: identifier, setter: setter };
    } else {
      var getter = args[0];
      var _setter = args[1];

      (0, _invariant2.default)(typeof getter === 'function', 'Store: `getter` is expected to be a function.');
      (0, _invariant2.default)(typeof _setter === 'function', 'Store: `setter` is expected to be a function.');

      dependency = { identifier: identifier, getter: getter, setter: _setter };
    }

    dependencies[identifier] = dependency;
    return this;
  };

  Store.prototype.post = function post(transformer) {
    (0, _invariant2.default)(typeof transformer === 'function', 'Store: `transformer` is expected to be a function.');

    this.hooks.post = transformer;
    return this;
  };

  Store.prototype.observeMissing = function observeMissing() {
    return this[missing].subject.asObservable();
  };

  Store.prototype.useCollection = function useCollection(col) {
    (0, _invariant2.default)(col && col.prototype instanceof _Collection.Collection, 'Store: `col` is expected to be a Collection.');

    this.collection = col;
    return this;
  };

  Store.prototype.getIdentifier = function getIdentifier() {
    return this.identifier;
  };

  Store.prototype.getPre = function getPre() {
    return this.hooks.pre || function (x) {
      return (0, _toMap2.default)(x);
    };
  };

  Store.prototype.getDependencies = function getDependencies() {
    return this.dependencies;
  };

  Store.prototype.getPost = function getPost() {
    return this.hooks.post;
  };

  Store.prototype.getReducer = function getReducer() {
    if (this.reducer && typeof this.reducer === 'function') {
      return this.reducer;
    }

    this.reducer = (0, _createReducerForStore2.default)(this);
    return this.reducer;
  };

  Store.prototype.createCollection = function createCollection() {
    var CollectionClass = this.collection || _Collection.Collection;

    var res = new CollectionClass();
    res.dependencies = (0, _keys2.default)(this.dependencies);

    return res;
  };

  Store.prototype._missing = function _missing(ids) {
    var identifier = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
    var _missing2 = this[missing];
    var cache = _missing2.cache;
    var subject = _missing2.subject;


    (0, _invariant2.default)(_immutable.Set.isSet(ids) && ids.every(function (x) {
      return typeof x === 'string';
    }), 'Store: `ids` is expected to be an Immutable.Set of ids (string).');

    cache[identifier] = ids;

    var missingIds = (0, _keys2.default)(cache).reduce(function (acc, key) {
      var _set = cache[key];
      return _set ? acc.union(_set) : acc;
    }, new _immutable.Set());

    subject.next(missingIds);
  };

  Store.prototype.insert = function insert(payload, groupId) {
    (0, _invariant2.default)(payload && (_immutable.Iterable.isKeyed(payload) || _immutable.Iterable.isIterable(payload) && payload.every && payload.every(_immutable.Iterable.isKeyed)), 'Store: `payload` is expected to be a keyed iterable or an iterable containing keyed iterables.');
    var identifier = this.identifier;


    return {
      type: _StoreConstants.STORE_INSERT,
      identifier: identifier,
      payload: payload,
      groupId: groupId
    };
  };

  Store.prototype.remove = function remove(payload) {
    (0, _invariant2.default)(payload && (typeof payload === 'string' || _immutable.Iterable.isKeyed(payload) && typeof payload.get('id') === 'string'), 'Store: `payload` is expected to be an id or a keyed iterable containing an id.');
    var identifier = this.identifier;


    return {
      type: _StoreConstants.STORE_REMOVE,
      identifier: identifier,
      payload: payload
    };
  };

  Store.prototype.filter = function filter(selector) {
    (0, _invariant2.default)(typeof selector === 'function', 'Store: `selector` is expected to be a function.');
    var identifier = this.identifier;


    return {
      type: _StoreConstants.STORE_FILTER,
      identifier: identifier,
      selector: selector
    };
  };

  Store.prototype.update = function update(selector) {
    (0, _invariant2.default)(typeof selector === 'function', 'Store: `selector` is expected to be a function.');
    var identifier = this.identifier;


    return {
      type: _StoreConstants.STORE_UPDATE,
      identifier: identifier,
      selector: selector
    };
  };

  return Store;
}();

function createStore(identifier, opts) {
  return new Store(identifier, opts);
}